@@ -5,7 +5,7 @@
 /*	Postfix lookup table management
 /* SYNOPSIS
 /* .fi
-/*	\fBpostmap\fR [\fB-Nbfhimnoprsvw\fR] [\fB-c \fIconfig_dir\fR]
+/*	\fBpostmap\fR [\fB-NbfhimnoprsuUvw\fR] [\fB-c \fIconfig_dir\fR]
 /*	[\fB-d \fIkey\fR] [\fB-q \fIkey\fR]
 /*		[\fIfile_type\fR:]\fIfile_name\fR ...
 /* DESCRIPTION
@@ -71,6 +71,11 @@
 /*	generates no body-style lookup keys for attachment MIME
 /*	headers and for attached message/* headers.
 /* .sp
+/*	NOTE: with "smtputf8_enable = yes", the \fB-b\fR option
+/*	option disables UTF-8 syntax checks on query keys and
+/*	lookup results. Specify the \fB-U\fR option to force UTF-8
+/*	syntax checks anyway.
+/* .sp
 /*	This feature is available in Postfix version 2.6 and later.
 /* .IP "\fB-c \fIconfig_dir\fR"
 /*	Read the \fBmain.cf\fR configuration file in the named directory
@@ -104,6 +109,11 @@
 /*	generates header-style lookup keys for attachment MIME
 /*	headers and for attached message/* headers.
 /* .sp
+/*	NOTE: with "smtputf8_enable = yes", the \fB-b\fR option
+/*	option disables UTF-8 syntax checks on query keys and
+/*	lookup results. Specify the \fB-U\fR option to force UTF-8
+/*	syntax checks anyway.
+/* .sp
 /*	This feature is available in Postfix version 2.6 and later.
 /* .IP \fB-i\fR
 /*	Incremental mode. Read entries from standard input and do not
@@ -151,6 +161,13 @@
 /* .sp
 /*	This feature is available in Postfix version 2.2 and later,
 /*	and is not available for all database types.
+/* .IP \fB-u\fR
+/*	Disable UTF-8 support. UTF-8 support is enabled by default
+/*	when "smtputf8_enable = yes". It requires that keys and
+/*	values are valid UTF-8 strings.
+/* .IP \fB-U\fR
+/*	With "smtputf8_enable = yes", force UTF-8 syntax checks
+/*	with the \fB-b\fR and \fB-h\fR options.
 /* .IP \fB-v\fR
 /*	Enable verbose logging for debugging purposes. Multiple \fB-v\fR
 /*	options make the software increasingly verbose.
@@ -229,6 +246,9 @@
 /* .IP "\fBdefault_database_type (see 'postconf -d' output)\fR"
 /*	The default database type for use in \fBnewaliases\fR(1), \fBpostalias\fR(1)
 /*	and \fBpostmap\fR(1) commands.
+/* .IP "\fBsmtputf8_enable (yes)\fR"
+/*	Enable preliminary SMTPUTF8 support for the protocols described
+/*	in RFC 6531..6533.
 /* .IP "\fBsyslog_facility (mail)\fR"
 /*	The syslog facility of Postfix logging.
 /* .IP "\fBsyslog_name (see 'postconf -d' output)\fR"
@@ -332,6 +352,7 @@ static void postmap(char *map_type, char *path_name, int postmap_flags,
     VSTRING *line_buffer;
     MKMAP  *mkmap;
     int     lineno;
+    int     last_line;
     char   *key;
     char   *value;
     struct stat st;
@@ -344,7 +365,7 @@ static void postmap(char *map_type, char *path_name, int postmap_flags,
     if ((open_flags & O_TRUNC) == 0) {
 	/* Incremental mode. */
 	source_fp = VSTREAM_IN;
-	vstream_control(source_fp, VSTREAM_CTL_PATH, "stdin", VSTREAM_CTL_END);
+	vstream_control(source_fp, CA_VSTREAM_CTL_PATH("stdin"), CA_VSTREAM_CTL_END);
     } else {
 	/* Create database. */
 	if (strcmp(map_type, DICT_TYPE_PROXY) == 0)
@@ -397,15 +418,27 @@ static void postmap(char *map_type, char *path_name, int postmap_flags,
 	/*
 	 * Add records to the database.
 	 */
-	lineno = 0;
-	while (readlline(line_buffer, source_fp, &lineno)) {
+	last_line = 0;
+	while (readllines(line_buffer, source_fp, &last_line, &lineno)) {
+
+	    /*
+	     * First some UTF-8 checks sans casefolding.
+	     */
+	    if ((mkmap->dict->flags & DICT_FLAG_UTF8_ACTIVE)
+		&& !allascii(STR(line_buffer))
+		&& !valid_utf8_string(STR(line_buffer), LEN(line_buffer))) {
+		msg_warn("%s, line %d: non-UTF-8 input \"%s\""
+			 " -- ignoring this line",
+			 VSTREAM_PATH(source_fp), lineno, STR(line_buffer));
+		continue;
+	    }
 
 	    /*
 	     * Split on the first whitespace character, then trim leading and
 	     * trailing whitespace from key and value.
 	     */
 	    key = STR(line_buffer);
-	    value = key + strcspn(key, " \t\r\n");
+	    value = key + strcspn(key, CHARS_SPACE);
 	    if (*value)
 		*value++ = 0;
 	    while (ISSPACE(*value))
@@ -622,7 +655,7 @@ static int postmap_queries(VSTREAM *in, char **maps, const int map_count,
     for (n = 0; n < map_count; n++)
 	if (dicts[n])
 	    dict_close(dicts[n]);
-    myfree((char *) dicts);
+    myfree((void *) dicts);
     vstring_free(keybuf);
 
     return (found);
@@ -704,7 +737,7 @@ static int postmap_deletes(VSTREAM *in, char **maps, const int map_count,
     for (n = 0; n < map_count; n++)
 	if (dicts[n])
 	    dict_close(dicts[n]);
-    myfree((char *) dicts);
+    myfree((void *) dicts);
     vstring_free(keybuf);
 
     return (found);
@@ -768,7 +801,7 @@ static void postmap_seq(const char *map_type, const char *map_name,
 
 static NORETURN usage(char *myname)
 {
-    msg_fatal("usage: %s [-Nfinoprsvw] [-c config_dir] [-d key] [-q key] [map_type:]file...",
+    msg_fatal("usage: %s [-NfinoprsuUvw] [-c config_dir] [-d key] [-q key] [map_type:]file...",
 	      myname);
 }
 
@@ -783,11 +816,13 @@ int     main(int argc, char **argv)
     struct stat st;
     int     postmap_flags = POSTMAP_FLAG_AS_OWNER | POSTMAP_FLAG_SAVE_PERM;
     int     open_flags = O_RDWR | O_CREAT | O_TRUNC;
-    int     dict_flags = DICT_FLAG_DUP_WARN | DICT_FLAG_FOLD_FIX;
+    int     dict_flags = (DICT_FLAG_DUP_WARN | DICT_FLAG_FOLD_FIX
+			  | DICT_FLAG_UTF8_REQUEST);
     char   *query = 0;
     char   *delkey = 0;
     int     sequence = 0;
     int     found;
+    int     force_utf8 = 0;
 
     /*
      * Fingerprint executables and core dumps.
@@ -833,7 +868,7 @@ int     main(int argc, char **argv)
     /*
      * Parse JCL.
      */
-    while ((ch = GETOPT(argc, argv, "Nbc:d:fhimnopq:rsvw")) > 0) {
+    while ((ch = GETOPT(argc, argv, "Nbc:d:fhimnopq:rsuUvw")) > 0) {
 	switch (ch) {
 	default:
 	    usage(argv[0]);
@@ -890,6 +925,12 @@ int     main(int argc, char **argv)
 		msg_fatal("specify only one of -s or -q or -d");
 	    sequence = 1;
 	    break;
+	case 'u':
+	    dict_flags &= ~DICT_FLAG_UTF8_REQUEST;
+	    break;
+	case 'U':
+	    force_utf8 = 1;
+	    break;
 	case 'v':
 	    msg_verbose++;
 	    break;
@@ -910,6 +951,9 @@ int     main(int argc, char **argv)
 	&& (postmap_flags & POSTMAP_FLAG_ANY_KEY)
 	== (postmap_flags & POSTMAP_FLAG_MIME_KEY))
 	msg_warn("ignoring -m option without -b or -h");
+    if ((postmap_flags & (POSTMAP_FLAG_ANY_KEY & ~POSTMAP_FLAG_MIME_KEY))
+	&& force_utf8 == 0)
+	dict_flags &= ~DICT_FLAG_UTF8_MASK;
 
     /*
      * Use the map type specified by the user, or fall back to a default
