@@ -6,15 +6,15 @@
 /* SYNOPSIS
 /*	#include <mymalloc.h>
 /*
-/*	char	*mymalloc(len)
+/*	void	*mymalloc(len)
 /*	ssize_t	len;
 /*
-/*	char	*myrealloc(ptr, len)
-/*	char	*ptr;
+/*	void	*myrealloc(ptr, len)
+/*	void	*ptr;
 /*	ssize_t	len;
 /*
 /*	void	myfree(ptr)
-/*	char	*ptr;
+/*	void	*ptr;
 /*
 /*	char	*mystrdup(str)
 /*	const char *str;
@@ -142,9 +142,9 @@ static const char empty_string[] = "";
 
 /* mymalloc - allocate memory or bust */
 
-char   *mymalloc(ssize_t len)
+void   *mymalloc(ssize_t len)
 {
-    char   *ptr;
+    void   *ptr;
     MBLOCK *real_ptr;
 
     /*
@@ -158,7 +158,8 @@ char   *mymalloc(ssize_t len)
     len += MYMALLOC_FUZZ;
 #endif
     if ((real_ptr = (MBLOCK *) malloc(SPACE_FOR(len))) == 0)
-	msg_fatal("mymalloc: insufficient memory: %m");
+	msg_fatal("mymalloc: insufficient memory for %ld bytes: %m",
+		  (long) len);
     CHECK_OUT_PTR(ptr, real_ptr, len);
     memset(ptr, FILLER, len);
     return (ptr);
@@ -166,7 +167,7 @@ char   *mymalloc(ssize_t len)
 
 /* myrealloc - reallocate memory or bust */
 
-char   *myrealloc(char *ptr, ssize_t len)
+void   *myrealloc(void *ptr, ssize_t len)
 {
     MBLOCK *real_ptr;
     ssize_t old_len;
@@ -187,8 +188,9 @@ char   *myrealloc(char *ptr, ssize_t len)
     len += MYMALLOC_FUZZ;
 #endif
     CHECK_IN_PTR(ptr, real_ptr, old_len, "myrealloc");
-    if ((real_ptr = (MBLOCK *) realloc((char *) real_ptr, SPACE_FOR(len))) == 0)
-	msg_fatal("myrealloc: insufficient memory: %m");
+    if ((real_ptr = (MBLOCK *) realloc((void *) real_ptr, SPACE_FOR(len))) == 0)
+	msg_fatal("myrealloc: insufficient memory for %ld bytes: %m",
+		  (long) len);
     CHECK_OUT_PTR(ptr, real_ptr, len);
     if (len > old_len)
 	memset(ptr + old_len, FILLER, len - old_len);
@@ -197,7 +199,7 @@ char   *myrealloc(char *ptr, ssize_t len)
 
 /* myfree - release memory */
 
-void    myfree(char *ptr)
+void    myfree(void *ptr)
 {
     MBLOCK *real_ptr;
     ssize_t len;
@@ -206,8 +208,8 @@ void    myfree(char *ptr)
     if (ptr != empty_string) {
 #endif
 	CHECK_IN_PTR(ptr, real_ptr, len, "myfree");
-	memset((char *) real_ptr, FILLER, SPACE_FOR(len));
-	free((char *) real_ptr);
+	memset((void *) real_ptr, FILLER, SPACE_FOR(len));
+	free((void *) real_ptr);
 #ifndef NO_SHARED_EMPTY_STRINGS
     }
 #endif
@@ -250,7 +252,7 @@ char   *mystrndup(const char *str, ssize_t len)
 
 /* mymemdup - copy memory */
 
-char   *mymemdup(const char *ptr, ssize_t len)
+char   *mymemdup(const void *ptr, ssize_t len)
 {
     if (ptr == 0)
 	msg_panic("mymemdup: null pointer argument");
